@@ -15,9 +15,10 @@ from plotly import tools, subplots
 import plotly.graph_objs as go
 from plotly.colors import DEFAULT_PLOTLY_COLORS
 from plotly.offline import iplot
+import matplotlib.pyplot as plt
 
 # Project imports
-from groundhog.general.plotting import plot_with_log, GROUNDHOG_PLOTTING_CONFIG
+from groundhog.general.plotting import plot_with_log, LogPlotMatplotlib, GROUNDHOG_PLOTTING_CONFIG
 from groundhog.general.parameter_mapping import offsets, latlon_distance
 
 
@@ -320,18 +321,21 @@ class SoilProfile(pd.DataFrame):
         x = np.insert(second_array_x, np.arange(len(first_array_x)), first_array_x)
         return z, x
 
-    def map_soilprofile(self, nodalcoords, target_depthkey='z [m]', keys_to_map=None, invert_sign=False, offset=0):
+    def map_soilprofile(self, nodalcoords, target_depthkey='z [m]', keys_to_map=None, invert_sign=False, offset=0,
+        include_layertransitions=False):
         """
         Maps the soilprofile to a grid. The depth coordinates to the grid are specified
         in a list or Numpy array (``nodalcoords``).
         The depth coordinates should be strictly ascending (no duplicates) and the minimum and
         maximum should be contained inside the soil profile bounds.
+        Layer transitions can be included in the grid when ``include_layertransitions=True``.
         All soil parameters are interpolated onto this grid.
         :param nodalcoords: List or Numpy array with the nodal coordinates of the grid
         :param target_depthkey: Name of the depth key in the resulting dataframe
         :param keys_to_map: List with the soilprofile keys to map
         :param invert_sign: Boolean determining whether to invert the sign after interpolation
         :param offset: Offset by which the depth is shifted (added to the depth after sign conversion)
+        :param include_layertransitions: Boolean to determine whether the layer transitions needs to be included in the grid (default=False)
         :return: Returns a dataframe with the full grid with soil parameters
         """
         # 1. Convert nodalcoords to Numpy array
@@ -339,6 +343,12 @@ class SoilProfile(pd.DataFrame):
             z = np.array(nodalcoords)
         except Exception as err:
             raise ValueError("Could not convert nodal coords to Numpy array (%s)" % str(err))
+        # Merge layer transitions
+        if include_layertransitions:
+            z = np.unique((np.append(z, self.layer_transitions())))
+        else:
+            pass
+
         # Create a target dataframe with the depth column
         target_df = pd.DataFrame({
             target_depthkey: z,
@@ -1170,4 +1180,62 @@ def plot_fence_diagram(
         if showfig:
             iplot(fig, filename='longitudinalplot', config=GROUNDHOG_PLOTTING_CONFIG)
 
-        return fig
\ No newline at end of file
+        return fig
+
+
+class CalculationGrid(object):
+    """
+    A ``CalculationGrid`` is an object which consist of a dataframe with nodes ``.nodes``
+    and a dataframe with elements ``.elements``.
+    Properties of the soil profile are mapped to the nodes and the elements to allow subsequent calculation.
+    At layer transition nodes, the properties of the layer below are always assigned to the node.
+    """
+    
+    def __init__(self, soilprofile, dz, custom_nodes=None, include_layertransitions=True):
+        """
+        Initializes the ``CalculationGrid`` object from a ``SoilProfile`` object.
+        A nodes offset ``dz`` needs to be specified. Additional nodes are inserted
+        at layer transitions by default (``include_layertransitions=True``).
+        The user can also specify a NumPy array with custom_nodes (None by default).
+        After initialization, the dataframes of nodal and elemental information are created.
+        For properties varying linearly across a layer, the value of the property at the element center
+        is also calculated and is given the name of the parameter (without from and to).
+        """
+        self.soilprofile = soilprofile
+        self.set_nodes(dz=dz, custom_nodes=custom_nodes, include_layertransitions=include_layertransitions)
+        self.set_elements()
+        
+    def set_nodes(self, dz, custom_nodes=None, include_layertransitions=True, **kwargs):
+        if custom_nodes is None:
+            no_nodes = int(np.ceil(1 + (self.soilprofile.max_depth - self.soilprofile.min_depth) / dz))
+            self.nodes = self.soilprofile.map_soilprofile(
+                nodalcoords=np.linspace(self.soilprofile.min_depth, self.soilprofile.max_depth, no_nodes),
+                include_layertransitions=include_layertransitions, **kwargs)
+        else:
+            self.nodes = self.soilprofile.map_soilprofile(
+                nodalcoords=custom_nodes,
+                include_layertransitions=include_layertransitions,
+                **kwargs)
+            
+    def set_elements(self):
+        self.elements = pd.DataFrame({
+            "z from [m]": list(self.nodes['z [m]'][:-1]),
+            "z to [m]": list(self.nodes['z [m]'][1:])
+        })
+        self.elements['z [m]'] = 0.5 * (
+            self.elements['z from [m]'] +
+            self.elements['z to [m]']
+            )
+        self.elements['dz [m]'] = list(self.nodes['z [m]'].diff()[1:])
+        for _param in self.soilprofile.numerical_soil_parameters():
+            if self.soilprofile.check_linear_variation(_param):
+                self.elements["%sfrom [%s" % (re.split('\[', _param)[0], re.split('\[', _param)[1])] = \
+                    list(self.nodes[_param])[:-1]
+                self.elements["%sto [%s" % (re.split('\[', _param)[0], re.split('\[', _param)[1])] = \
+                    list(self.nodes[_param])[1:]
+                self.elements[_param] = \
+                    0.5 * (np.array(list(self.nodes[_param])[:-1]) +
+                           np.array(list(self.nodes[_param])[1:]))
+            else:
+                self.elements[_param] = list(self.nodes[_param][:-1])
+
