@@ -90,6 +90,16 @@ class Test_SoilProfile(unittest.TestCase):
         self.assertEqual(self.profile.layer_transitions()[0], 1)
         self.assertEqual(self.profile.layer_transitions(include_top=True)[0], 0)
 
+    def test_profile_depthreference_modification(self):
+        self.profile.convert_depth_reference(newname='Depth', newunit='ft', multiplier=1/0.3048)
+        self.assertAlmostEqual(self.profile.max_depth, 20 / 0.3048, 3)
+        self.assertEqual(self.profile.min_depth, 0)
+        self.assertAlmostEqual(self.profile.layer_transitions()[0], 1 / 0.3048, 3)
+        self.profile.convert_depth_reference(newname='Depth', newunit='m', multiplier=0.3048)
+        self.assertAlmostEqual(self.profile.max_depth, 20, 3)
+        self.assertEqual(self.profile.min_depth, 0)
+        self.assertAlmostEqual(self.profile.layer_transitions()[0], 1, 3)
+
     def test_soilparameter_retrieval(self):
         profile = sp.SoilProfile({
             'Depth from [m]': [0, 1, 5, 10],
@@ -288,8 +298,8 @@ class Test_SoilProfile(unittest.TestCase):
             'qc to [MPa]': [2, 4, 20, 50]
         })
         profile.calculate_overburden(waterlevel=4)
-        self.assertEqual(profile['Total vertical stress to [kPa]'].iloc[-1], 376)
-        self.assertEqual(profile['Effective vertical stress to [kPa]'].iloc[-1], 216)
+        self.assertEqual(profile['Vertical total stress to [kPa]'].iloc[-1], 376)
+        self.assertEqual(profile['Vertical effective stress to [kPa]'].iloc[-1], 216)
         self.assertEqual(profile['Hydrostatic pressure to [kPa]'].iloc[-1], 160)
 
     def test_applyfunction(self):
@@ -343,4 +353,49 @@ class Test_SoilProfile(unittest.TestCase):
             'Depth to [m]': [0, 2, 4, 6],
             'Soil type': ['SAND', 'CLAY', 'SAND', 'CLAY']
         })
-        sp.profile_from_dataframe(df[1:])
\ No newline at end of file
+        sp.profile_from_dataframe(df[1:])
+
+    def test_parameter_at_depth(self):
+        """
+        Check whether the routine for selection of parameters at a given depth works
+        :return:
+        """
+        profile = sp.SoilProfile({
+            'Depth from [m]': [0, 1, 5, 10],
+            'Depth to [m]': [1, 5, 10, 20],
+            'Soil type': ['SAND', 'SILT', 'CLAY', 'SAND'],
+            'Relative density': ['Loose', 'Medium dense', None, 'Dense'],
+            'Total unit weight [kN/m3]': [19, 18, 17, 20],
+            'Dr [%]': [40, 60, np.nan, 80],
+            'qc from [MPa]': [1, np.nan, 10, 40],
+            'qc to [MPa]': [2, np.nan, 10, 50]
+        })
+        # String parameter
+        self.assertEqual(
+            profile.parameter_at_depth(depth=0.5, parameter='Soil type'), 'SAND')
+        self.assertEqual(
+            profile.parameter_at_depth(depth=3, parameter='Soil type'), 'SILT')
+        self.assertEqual(
+            profile.parameter_at_depth(depth=5, parameter='Soil type'), 'SILT')
+        self.assertEqual(
+            profile.parameter_at_depth(depth=5, parameter='Soil type', shallowest=False), 'CLAY')
+        # Errors when out of bounds
+        self.assertRaises(
+            ValueError, profile.parameter_at_depth, -1, 'Soil type')
+        self.assertRaises(
+            ValueError, profile.parameter_at_depth, 100, 'Soil type')
+        # Errors when parameter not available
+        self.assertRaises(
+            ValueError, profile.parameter_at_depth, 12, 'Color')
+        # Test Numerical soil parameter with constant value
+        self.assertEqual(
+            profile.parameter_at_depth(depth=3, parameter='Total unit weight [kN/m3]'), 18)
+        self.assertEqual(
+            profile.parameter_at_depth(depth=13, parameter='Dr [%]'), 80)
+        # Test Numerical soil parameter with linear variation
+        self.assertEqual(
+            profile.parameter_at_depth(depth=0.5, parameter='qc [MPa]'), 1.5)
+        self.assertEqual(
+            profile.parameter_at_depth(depth=7, parameter='qc [MPa]'), 10)
+        self.assertEqual(
+            profile.parameter_at_depth(depth=15, parameter='qc [MPa]'), 45)
